@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sw=4 et tw=80:
  *
  * ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
@@ -41,10 +42,7 @@
 /* JavaScript JSClasses and JSOps for our Wrapped Native JS Objects. */
 
 #include "xpcprivate.h"
-
-// Pull in nsIScriptSecurityManager.h temporarily until we can do
-// IsSystemCallingContent() w/o a direct caps dependency.
-#include "nsIScriptSecurityManager.h"
+#include "XPCNativeWrapper.h"
 
 /***************************************************************************/
 
@@ -121,7 +119,7 @@ ToStringGuts(XPCCallContext& ccx)
 
 /***************************************************************************/
 
-JS_STATIC_DLL_CALLBACK(JSBool)
+JSBool JS_DLL_CALLBACK
 XPC_WN_Shared_ToString(JSContext *cx, JSObject *obj,
                        uintN argc, jsval *argv, jsval *vp)
 {
@@ -246,8 +244,6 @@ XPC_WN_DoubleWrappedGetter(JSContext *cx, JSObject *obj,
  * We *never* set toString or toSource as JS_ENUMERATE.
  */
 
-#define JSPROP_TRUST_ME 0x100   // XXXbe secret handshake
-
 static JSBool
 DefinePropertyIfFound(XPCCallContext& ccx,
                       JSObject *obj, jsval idval,
@@ -267,8 +263,6 @@ DefinePropertyIfFound(XPCCallContext& ccx,
     const char* name;
     jsid id;
 
-    propFlags |= JSPROP_TRUST_ME;
-
     if(set)
     {
         if(iface)
@@ -387,13 +381,13 @@ DefinePropertyIfFound(XPCCallContext& ccx,
         }
 
 #ifdef XPC_IDISPATCH_SUPPORT
-        // Check to see if there's an IDispatch tearoff
+        // Check to see if there's an IDispatch tearoff     
         if(wrapperToReflectInterfaceNames &&
-            XPCIDispatchExtension::DefineProperty(ccx, obj,
+            XPCIDispatchExtension::DefineProperty(ccx, obj, 
                 idval, wrapperToReflectInterfaceNames, propFlags, resolved))
             return JS_TRUE;
 #endif
-
+        
         if(resolved)
             *resolved = JS_FALSE;
         return JS_TRUE;
@@ -438,20 +432,22 @@ DefinePropertyIfFound(XPCCallContext& ccx,
                                    propFlags, nsnull);
     }
 
-    if(scriptableInfo &&
-       scriptableInfo->GetFlags().DontEnumQueryInterface() &&
-       idval == rt->GetStringJSVal(XPCJSRuntime::IDX_QUERY_INTERFACE))
+    if(idval == rt->GetStringJSVal(XPCJSRuntime::IDX_TO_STRING) ||
+       idval == rt->GetStringJSVal(XPCJSRuntime::IDX_TO_SOURCE) ||
+       (scriptableInfo &&
+        scriptableInfo->GetFlags().DontEnumQueryInterface() &&
+        idval == rt->GetStringJSVal(XPCJSRuntime::IDX_QUERY_INTERFACE)))
         propFlags &= ~JSPROP_ENUMERATE;
 
     JSObject* funobj;
-
+    
     {
         // scoped gc protection of funval
         jsval funval;
 
         if(!member->GetValue(ccx, iface, &funval))
             return JS_FALSE;
-
+    
         AUTO_MARK_JSVAL(ccx, funval);
 
         funobj = xpc_CloneJSFunction(ccx, JSVAL_TO_OBJECT(funval), obj);
@@ -680,7 +676,7 @@ xpc_MarkForValidWrapper(JSContext *cx, XPCWrappedNative* wrapper, void *arg)
     // here is because the bits used in that marking do unpleasant things to the
     // member counts in the interface and interface set objects. Those counts
     // are used in the DealWithDyingGCThings calls that are part of this JS GC
-    // marking phase. By doing these calls later during our GC callback we
+    // marking phase. By doing these calls later during our GC callback we 
     // avoid that problem. Arguably this could be changed. But it ain't broke.
 
     // However, we do need to call the wrapper's MarkBeforeJSFinalize so that
@@ -694,7 +690,7 @@ xpc_MarkForValidWrapper(JSContext *cx, XPCWrappedNative* wrapper, void *arg)
     // need to use the JSClass. So, some marking is required for protection.
 
     wrapper->MarkBeforeJSFinalize(cx);
-
+     
     if(wrapper->HasProto())
     {
         JSObject* obj = wrapper->GetProto()->GetJSProtoObject();
@@ -740,31 +736,161 @@ XPC_WN_NoHelper_Resolve(JSContext *cx, JSObject *obj, jsval idval)
                                  JSPROP_PERMANENT, nsnull);
 }
 
-JSClass XPC_WN_NoHelper_JSClass = {
-    "XPCWrappedNative_NoHelper",    // name;
-    JSCLASS_HAS_PRIVATE |
-    JSCLASS_PRIVATE_IS_NSISUPPORTS, // flags;
+nsISupports *
+GetIdentityObject(JSContext *cx, JSObject *obj)
+{
+    XPCWrappedNative *wrapper;
 
-    /* Mandatory non-null function pointer members. */
-    XPC_WN_OnlyIWrite_PropertyStub, // addProperty;
-    XPC_WN_CannotModifyPropertyStub,// delProperty;
-    JS_PropertyStub,                // getProperty;
-    XPC_WN_OnlyIWrite_PropertyStub, // setProperty;
+    if(XPCNativeWrapper::IsNativeWrapper(cx, obj))
+        wrapper = XPCNativeWrapper::GetWrappedNative(cx, obj);
+    else
+        wrapper = XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj);
 
-    XPC_WN_Shared_Enumerate,        // enumerate;
-    XPC_WN_NoHelper_Resolve,        // resolve;
-    XPC_WN_Shared_Convert,          // convert;
-    XPC_WN_NoHelper_Finalize,       // finalize;
+    if(!wrapper) {
+        return nsnull;
+    }
 
-    /* Optionally non-null members start here. */
-    nsnull,                         // getObjectOps;
-    nsnull,                         // checkAccess;
-    nsnull,                         // call;
-    nsnull,                         // construct;
-    nsnull,                         // xdrObject;
-    nsnull,                         // hasInstance;
-    XPC_WN_Shared_Mark,             // mark;
-    nsnull                          // spare;
+    return wrapper->GetIdentityObject();
+}
+
+JS_STATIC_DLL_CALLBACK(JSBool)
+XPC_WN_Equality(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
+{
+    *bp = JS_FALSE;
+
+    XPCWrappedNative *wrapper =
+        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj);
+    THROW_AND_RETURN_IF_BAD_WRAPPER(cx, wrapper);
+
+    XPCNativeScriptableInfo* si = wrapper->GetScriptableInfo();
+    if(si && si->GetFlags().WantEquality())
+    {
+        nsresult rv = si->GetCallback()->Equality(wrapper, cx, obj, v, bp);
+
+        if(NS_FAILED(rv))
+            return Throw(rv, cx);
+    }
+    else if(!JSVAL_IS_PRIMITIVE(v))
+    {
+        JSObject *other = JSVAL_TO_OBJECT(v);
+
+        *bp = (obj == other ||
+               GetIdentityObject(cx, obj) == GetIdentityObject(cx, other));
+    }
+
+    return JS_TRUE;
+}
+
+JS_STATIC_DLL_CALLBACK(JSObject *)
+XPC_WN_OuterObject(JSContext *cx, JSObject *obj)
+{
+    XPCWrappedNative *wrapper =
+        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj);
+    if(!wrapper)
+    {
+        Throw(NS_ERROR_XPC_BAD_OP_ON_WN_PROTO, cx);
+
+        return nsnull;
+    }
+
+    if(!wrapper->IsValid())
+    {
+        Throw(NS_ERROR_XPC_HAS_BEEN_SHUTDOWN, cx);
+
+        return nsnull;
+    }
+
+    XPCNativeScriptableInfo* si = wrapper->GetScriptableInfo();
+    if(si && si->GetFlags().WantOuterObject())
+    {
+        JSObject *newThis;
+        nsresult rv =
+            si->GetCallback()->OuterObject(wrapper, cx, obj, &newThis);
+
+        if(NS_FAILED(rv))
+        {
+            Throw(rv, cx);
+
+            return nsnull;
+        }
+
+        obj = newThis;
+    }
+
+    return obj;
+}
+
+JS_STATIC_DLL_CALLBACK(JSObject *)
+XPC_WN_InnerObject(JSContext *cx, JSObject *obj)
+{
+    XPCWrappedNative *wrapper =
+        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj);
+    if(!wrapper)
+    {
+        Throw(NS_ERROR_XPC_BAD_OP_ON_WN_PROTO, cx);
+
+        return nsnull;
+    }
+
+    if(!wrapper->IsValid())
+    {
+        Throw(NS_ERROR_XPC_HAS_BEEN_SHUTDOWN, cx);
+
+        return nsnull;
+    }
+
+    XPCNativeScriptableInfo* si = wrapper->GetScriptableInfo();
+    if(si && si->GetFlags().WantInnerObject())
+    {
+        JSObject *newThis;
+        nsresult rv =
+            si->GetCallback()->InnerObject(wrapper, cx, obj, &newThis);
+
+        if(NS_FAILED(rv))
+        {
+            Throw(rv, cx);
+
+            return nsnull;
+        }
+
+        obj = newThis;
+    }
+
+    return obj;
+}
+
+JSExtendedClass XPC_WN_NoHelper_JSClass = {
+    {
+        "XPCWrappedNative_NoHelper",    // name;
+        JSCLASS_HAS_PRIVATE |
+        JSCLASS_PRIVATE_IS_NSISUPPORTS |
+        JSCLASS_IS_EXTENDED, // flags;
+
+        /* Mandatory non-null function pointer members. */
+        XPC_WN_OnlyIWrite_PropertyStub, // addProperty;
+        XPC_WN_CannotModifyPropertyStub,// delProperty;
+        JS_PropertyStub,                // getProperty;
+        XPC_WN_OnlyIWrite_PropertyStub, // setProperty;
+
+        XPC_WN_Shared_Enumerate,        // enumerate;
+        XPC_WN_NoHelper_Resolve,        // resolve;
+        XPC_WN_Shared_Convert,          // convert;
+        XPC_WN_NoHelper_Finalize,       // finalize;
+
+        /* Optionally non-null members start here. */
+        nsnull,                         // getObjectOps;
+        nsnull,                         // checkAccess;
+        nsnull,                         // call;
+        nsnull,                         // construct;
+        nsnull,                         // xdrObject;
+        nsnull,                         // hasInstance;
+        XPC_WN_Shared_Mark,             // mark;
+        nsnull                          // spare;
+    },
+    XPC_WN_Equality,
+    XPC_WN_OuterObject,
+    XPC_WN_InnerObject,
+    JSCLASS_NO_RESERVED_MEMBERS
 };
 
 
@@ -812,491 +938,14 @@ XPC_WN_Helper_DelProperty(JSContext *cx, JSObject *obj, jsval idval, jsval *vp)
     POST_HELPER_STUB
 }
 
-static nsIScriptSecurityManager *sSecMan;
-static nsIPrincipal *sSystemPrincipal;
-
-static PRBool
-EnsureSecMan()
-{
-    if(!sSecMan)
-    {
-        CallGetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &sSecMan);
-
-        // No security manager reachable, not much point in pretending
-        // we care about security then.
-        if(!sSecMan)
-            return PR_FALSE;
-
-        sSecMan->GetSystemPrincipal(&sSystemPrincipal);
-
-        if(!sSystemPrincipal)
-        {
-            // No system principal. Shouldn't happen, but if it does
-            // we've lost already.
-            NS_RELEASE(sSecMan);
-
-            return PR_FALSE;
-        }
-    }
-    return PR_TRUE;
-}
-
-void
-XPC_WN_JSOps_Shutdown()
-{
-    NS_IF_RELEASE(sSecMan);
-}
-
-class XPCWrappedNativeOrProto
-{
-public:
-#define XPC_PROTO_TAG     ((jsword)0x1)
-#define XPC_PROTO_WORD(p) ((jsword)(p))
-
-    static JSBool
-    IsTaggedProto(XPCWrappedNativeProto* p)
-        {return XPC_PROTO_WORD(p) & XPC_PROTO_TAG;}
-
-    static XPCWrappedNativeProto*
-    TagProto(XPCWrappedNativeProto* s)
-        {NS_ASSERTION(!IsTaggedProto(s), "bad pointer!");
-         return (XPCWrappedNativeProto*)(XPC_PROTO_WORD(s) | XPC_PROTO_TAG);}
-
-    static XPCWrappedNativeProto*
-    UntagProto(XPCWrappedNativeProto* s)
-        {return (XPCWrappedNativeProto*)(XPC_PROTO_WORD(s) & ~XPC_PROTO_TAG);}
-
-    JSBool
-    IsWrapper() const {return !IsTaggedProto(mMaybeProto);}
-
-    XPCWrappedNativeOrProto(XPCWrappedNative *wrapper)
-    {
-        NS_ASSERTION(wrapper, "oops, null wrapper");
-        mMaybeWrapper = wrapper;
-    }
-
-    XPCWrappedNativeOrProto(XPCWrappedNativeProto *proto)
-    {
-        NS_ASSERTION(proto, "oops, null proto");
-        mMaybeProto = TagProto(proto);
-    }
-
-    operator void*()
-        {return UntagProto(mMaybeProto);}
-
-    XPCWrappedNative* GetWrapper()
-        {NS_ASSERTION(IsWrapper(), "d'oh!"); return mMaybeWrapper;}
-
-    XPCWrappedNativeProto* GetProto()
-        {NS_ASSERTION(!IsWrapper(), "D'OH!"); return UntagProto(mMaybeProto);}
-
-    XPCWrappedNativeScope* GetScope()
-        {return IsWrapper()
-                ? mMaybeWrapper->GetScope()
-                : UntagProto(mMaybeProto)->GetScope();}
-
-    JSObject* GetFlatJSObject()
-        {return IsWrapper()
-                ? mMaybeWrapper->GetFlatJSObject()
-                : UntagProto(mMaybeProto)->GetJSProtoObject();}
-
-    XPCNativeScriptableInfo* GetScriptableInfo()
-        {return IsWrapper()
-                ? mMaybeWrapper->GetScriptableInfo()
-                : UntagProto(mMaybeProto)->GetScriptableInfo();}
-
-    static XPCWrappedNativeOrProto
-    GetWrappedNativeOfJSObject(JSContext *cx, JSObject *obj,
-                               JSObject *funobj = nsnull)
-    {
-        if(JS_InstanceOf(cx, obj, &XPC_WN_ModsAllowed_Proto_JSClass, nsnull))
-            return (XPCWrappedNativeProto*) JS_GetPrivate(cx, obj);
-
-        XPCWrappedNative* wrapper =
-            XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj, funobj);
-
-        // Inline and specialize THROW_AND_RETURN_IF_BAD_WRAPPER(cx, wrapper);
-        if(!wrapper)
-            Throw(NS_ERROR_XPC_BAD_OP_ON_WN_PROTO, cx);
-        else if(!wrapper->IsValid())
-        {
-            Throw(NS_ERROR_XPC_HAS_BEEN_SHUTDOWN, cx);
-            wrapper = nsnull;
-        }
-        return wrapper;
-    }
-
-private:
-    union
-    {
-        XPCWrappedNative*       mMaybeWrapper;
-        XPCWrappedNativeProto*  mMaybeProto;
-    };
-};
-
-// XXXbe, XXXjst this checks if our chrome code is calling into
-// content code, but perhaps it should check for any trust-domain
-// boundary crossing.
-static PRBool
-IsSystemCallingContent(JSContext *cx, XPCWrappedNativeOrProto wrapper_or_proto)
-{
-    // Provided script is active on cx, a scope chain that's the same
-    // as the global object in the wrappers scope means we're not in a
-    // cross-trust- domain call (i.e. our system code is not calling
-    // content etc).
-    if(cx->fp &&
-       (cx->fp->scopeChain == wrapper_or_proto.GetScope()->GetGlobalJSObject()))
-        return PR_FALSE;
-
-    if(!EnsureSecMan())
-        return PR_FALSE;
-
-    PRBool isCallerChrome = PR_FALSE;
-    if(NS_SUCCEEDED(sSecMan->SubjectPrincipalIsSystem(&isCallerChrome)) &&
-       isCallerChrome)
-    {
-        nsCOMPtr<nsIPrincipal> objectPrincipal;
-
-        sSecMan->GetObjectPrincipal(cx, wrapper_or_proto.GetFlatJSObject(),
-                                    getter_AddRefs(objectPrincipal));
-
-        return objectPrincipal != sSystemPrincipal;
-    }
-
-    return PR_FALSE;
-}
-
-static JSBool
-GetOrSetUnshadowedMemberValue(JSContext *cx,
-                              XPCWrappedNativeOrProto wrapper_or_proto,
-                              jsval idval, uintN argc, jsval *argv, jsval *vp,
-                              JSBool *foundMember)
-{
-    // Ok, we know chrome is getting a content-defined property on a
-    // wrapped native prototype.  We're done -- no way can we let this
-    // madness go further.
-    if(!wrapper_or_proto.IsWrapper())
-        return Throw(NS_ERROR_XPC_BAD_OP_ON_WN_PROTO, cx);
-
-    XPCWrappedNative* wrapper = wrapper_or_proto.GetWrapper();
-    JSObject *obj = wrapper->GetFlatJSObject();
-
-    // this will do verification and the method lookup for us
-    XPCCallContext ccx(JS_CALLER, cx, obj, nsnull, idval);
-
-    // did we find a method/attribute by that name?
-    XPCNativeMember* member = ccx.GetMember();
-    if(member)
-    {
-        *foundMember = JS_TRUE;
-
-        // it would a be a big surprise if there is a member without
-        // an interface :)
-        XPCNativeInterface* iface = ccx.GetInterface();
-        if(!iface)
-            return Throw(NS_ERROR_XPC_BAD_CONVERT_JS, cx);
-
-        // get (and perhaps lazily create) the member's cloneable
-        // value (which is a function, unless we're dealing with a
-        // constant).
-        jsval val;
-        if(!member->GetValue(ccx, iface, &val))
-            return Throw(NS_ERROR_XPC_BAD_CONVERT_JS, cx);
-
-        // For constants, we simply return the value
-        if(member->IsConstant())
-        {
-            *vp = val;
-            return JS_TRUE;
-        }
-
-        // Make sure val doesn't get collected during any of the JS
-        // engine calls in this scope from here on.
-        AUTO_MARK_JSVAL(ccx, val);
-
-        // XXXbe why did jband add this?  It's not strictly necessary in core
-        // JS native objects, e.g. d = new Date; y = d.getFullYear() does not
-        // clone Date.prototype.getFullYear solely in order to make
-        // d.getFullYear.__parent__ == d.
-        // See the log comment for 1.26.22.10 of this file.
-        // The concern here is that while jband cloned in Resolve, we are
-        // cloning on every Get that crosses from chrome to content.
-
-        // clone a function we can use for this object
-        JSObject* funobj =
-            xpc_CloneJSFunction(ccx, JSVAL_TO_OBJECT(val), obj);
-        if(!funobj)
-            return Throw(NS_ERROR_XPC_BAD_CONVERT_JS, cx);
-
-        // Extra scope just to be able to use AUTO_MARK_JSVAL once
-        // more for funval below. W/o the extra scope we get name
-        // collisions from the macro.
-        {
-            jsval funval = OBJECT_TO_JSVAL(funobj);
-
-            // protect funobj until it is actually attached
-            AUTO_MARK_JSVAL(ccx, funval);
-
-            // Call the getter function if the member is an attribute
-            if(member->IsAttribute())
-            {
-                jsid id;
-                if(!JS_ValueToId(cx, idval, &id))
-                    return JS_FALSE;
-
-                NS_ASSERTION(argc == 0 || member->IsWritableAttribute(),
-                             "Huh, setter called for readonly attribute!");
-
-                // Do the actual call to the getter or setter.
-                return js_InternalGetOrSet(cx, obj, id, funval,
-                                           (argc == 0) ? JSACC_READ : JSACC_WRITE,
-                                           argc, argv, vp);
-            }
-
-            // return the function
-            *vp = OBJECT_TO_JSVAL(funobj);
-        }
-    } else
-        *foundMember = JS_FALSE;
-
-    return JS_TRUE;
-}
-
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_WN_Helper_GetProperty(JSContext *cx, JSObject *obj, jsval idval, jsval *vp)
 {
-    // We can't use PRE_HELPER_STUB here since we need code between
-    // it and what it expects to follow.
-    XPCWrappedNative* wrapper =
-        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj);
-    THROW_AND_RETURN_IF_BAD_WRAPPER(cx, wrapper);
-
-    if(IsSystemCallingContent(cx, wrapper))
-    {
-        JSBool foundMember;
-        if(!GetOrSetUnshadowedMemberValue(cx, wrapper, idval, 0, nsnull, vp,
-                                          &foundMember))
-            return JS_FALSE;
-
-        if(foundMember)
-            return JS_TRUE;
-    }
-
-    PRBool retval = JS_TRUE;
-    nsresult rv = wrapper->GetScriptableCallback()->
-        GetProperty(wrapper, cx, obj, idval, vp, &retval);
+    PRE_HELPER_STUB
+    GetProperty(wrapper, cx, obj, idval, vp, &retval);
     POST_HELPER_STUB
 }
 
-JS_STATIC_DLL_CALLBACK(JSBool)
-XPC_WN_Safe_PropertyStub(JSContext *cx, JSObject *obj, jsval idval, jsval *vp)
-{
-    XPCWrappedNativeOrProto wrapper_or_proto =
-        XPCWrappedNativeOrProto::GetWrappedNativeOfJSObject(cx, obj);
-
-    if(wrapper_or_proto && IsSystemCallingContent(cx, wrapper_or_proto))
-    {
-        JSBool foundMember;
-        if(!GetOrSetUnshadowedMemberValue(cx, wrapper_or_proto, idval,
-                                          0, nsnull, vp, &foundMember))
-            return JS_FALSE;
-
-        if(foundMember)
-            return JS_TRUE;
-    }
-
-    return JS_PropertyStub(cx, obj, idval, vp);
-}
-
-// We must wrap untrusted (user-defined) getters and setters defined
-// on wrapped native instances with a thunk that checks whether the
-// caller is crossing a trust domain, and bypasses the user-defined
-// getter or setter if so.
-
-const char id_str[]             = "id";
-const char unsafe_gsobj_str[]   = "unsafe_gsobj";
-
-JS_STATIC_DLL_CALLBACK(JSBool)
-XPC_WN_Safe_GetterSetterThunkNative(JSContext *cx, JSObject *obj, uintN argc,
-                                    jsval *argv, jsval *rval)
-{
-    JSObject *safe_gsobj = JSVAL_TO_OBJECT(argv[-2]);
-
-    jsval idval;
-    if(!JS_GetProperty(cx, safe_gsobj, id_str, &idval))
-        return JS_FALSE;
-
-    jsid id;
-    if(!JS_ValueToId(cx, idval, &id))
-        return JS_FALSE;
-
-    XPCWrappedNativeOrProto wrapper_or_proto =
-        XPCWrappedNativeOrProto::GetWrappedNativeOfJSObject(cx, obj,
-                                                            safe_gsobj);
-
-    if(wrapper_or_proto && IsSystemCallingContent(cx, wrapper_or_proto))
-    {
-        JSBool foundMember;
-        if(!GetOrSetUnshadowedMemberValue(cx, wrapper_or_proto,
-                                          ID_TO_VALUE(id), argc, argv, rval,
-                                          &foundMember))
-            return JS_FALSE;
-
-        if(foundMember)
-            return JS_TRUE;
-
-        XPCWrappedNative* wrapper = wrapper_or_proto.GetWrapper();
-        XPCNativeScriptableInfo* si = wrapper->GetScriptableInfo();
-        if(si)
-        {
-            if(argc == 0 && si->GetFlags().WantGetProperty())
-            {
-                // wrapper wants GetProperty() calls, call it, to prevent
-                // user defined getters from shadowing things from chrome
-                // code.
-                JSBool ok = PR_TRUE;
-                nsresult rv =
-                    si->GetCallback()->GetProperty(wrapper, cx,
-                                                   wrapper->GetFlatJSObject(),
-                                                   id, rval, &ok);
-
-                if(NS_FAILED(rv))
-                    return Throw(rv, cx);
-
-                return ok;
-            }
-
-            if(argc == 1 && si->GetFlags().WantSetProperty())
-            {
-                // wrapper wants SetProperty() calls, call it, to prevent
-                // user defined setters from shadowing things from chrome
-                // code.
-                JSBool ok = PR_TRUE;
-                jsval v = argv[0];
-
-                nsresult rv =
-                    si->GetCallback()->SetProperty(wrapper, cx,
-                                                   wrapper->GetFlatJSObject(),
-                                                   id, &v, &ok);
-
-                if(NS_FAILED(rv))
-                    return Throw(rv, cx);
-
-                return ok;
-            }
-        }
-
-        // Fall through
-    }
-
-    jsval unsafe_gsobj_val;
-    if(!JS_GetProperty(cx, safe_gsobj, unsafe_gsobj_str, &unsafe_gsobj_val))
-        return JS_FALSE;
-
-    return js_InternalGetOrSet(cx, obj, id, unsafe_gsobj_val,
-                               (argc == 1) ? JSACC_WRITE : JSACC_READ,
-                               argc, argv, rval);
-}
-
-#include "jsfun.h"      // XXXbe should have a JS_GetFunctionNative API
-
-static JSPropertyOp
-NewSafeGetterOrSetterThunk(JSContext *cx, JSObject *obj, jsid id,
-                           JSPropertyOp gsop, uintN nargs)
-{
-    JSObject *unsafe_gsobj = (JSObject *) gsop;
-
-    // If called from native code, no need for a thunk: gsop is trusted.
-    if(!cx->fp)
-        return gsop;
-
-    // Similarly, check whether the caller is a chrome script we must trust.
-    if(!EnsureSecMan())
-        return NULL;
-    PRBool isCallerChrome = PR_FALSE;
-    if(NS_SUCCEEDED(sSecMan->SubjectPrincipalIsSystem(&isCallerChrome)) &&
-       isCallerChrome)
-        return gsop;
-
-    // Don't double-thunk if we can help it.
-    if(JS_ObjectIsFunction(cx, unsafe_gsobj))
-    {
-        JSFunction *unsafe_gsfun =
-            (JSFunction *) JS_GetPrivate(cx, unsafe_gsobj);
-        if(unsafe_gsfun->native == XPC_WN_Safe_GetterSetterThunkNative)
-            return gsop;
-    }
-
-    // Make a thunk to check cross-trust-domain access before calling gsop.
-    JSFunction *safe_gsfun = JS_NewFunction(cx,
-                                            XPC_WN_Safe_GetterSetterThunkNative,
-                                            nargs, 0, obj, NULL);
-    if(!safe_gsfun)
-        return NULL;
-
-    JSObject *safe_gsobj = JS_GetFunctionObject(safe_gsfun);
-
-    // The only clean way to get id and unsafe_gsobj through is via permanent,
-    // readonly properties on safe_gsobj.
-    if(!JS_DefineProperty(cx, safe_gsobj, id_str, ID_TO_VALUE(id), NULL, NULL,
-                          JSPROP_READONLY | JSPROP_PERMANENT))
-        return NULL;
-
-    if(!JS_DefineProperty(cx, safe_gsobj, unsafe_gsobj_str,
-                          OBJECT_TO_JSVAL(unsafe_gsobj), NULL, NULL,
-                          JSPROP_READONLY | JSPROP_PERMANENT))
-        return NULL;
-
-    return (JSPropertyOp) safe_gsobj;
-}
-
-JS_STATIC_DLL_CALLBACK(JSBool)
-XPC_WN_JSOp_Safe_DefineProperty(JSContext *cx, JSObject *obj,
-                                jsid id, jsval value,
-                                JSPropertyOp getter, JSPropertyOp setter,
-                                uintN attrs, JSProperty **propp)
-{
-    if(!(attrs & JSPROP_TRUST_ME))
-    {
-        if(attrs & JSPROP_GETTER)
-        {
-            getter = NewSafeGetterOrSetterThunk(cx, obj, id, getter, 0);
-            if(!getter)
-                return JS_FALSE;
-        }
-        if(attrs & JSPROP_SETTER)
-        {
-            setter = NewSafeGetterOrSetterThunk(cx, obj, id, setter, 1);
-            if(!setter)
-                return JS_FALSE;
-        }
-    }
-
-    return js_ObjectOps.defineProperty(cx, obj, id, value, getter, setter,
-                                       attrs, propp);
-}
-
-JS_STATIC_DLL_CALLBACK(JSBool)
-XPC_WN_JSOp_Safe_GetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
-{
-    XPCWrappedNativeOrProto wrapper_or_proto =
-        XPCWrappedNativeOrProto::GetWrappedNativeOfJSObject(cx, obj);
-
-    if(IsSystemCallingContent(cx, wrapper_or_proto))
-    {
-        JSBool foundMember;
-        if(!GetOrSetUnshadowedMemberValue(cx, wrapper_or_proto,
-                                          ID_TO_VALUE(id), 0, nsnull, vp,
-                                          &foundMember))
-            return JS_FALSE;
-
-        if(foundMember)
-            return JS_TRUE;
-    }
-
-    return js_ObjectOps.getProperty(cx, obj, id, vp);
-}
-
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_WN_Helper_SetProperty(JSContext *cx, JSObject *obj, jsval idval, jsval *vp)
 {
@@ -1478,7 +1127,6 @@ extern "C" JS_IMPORT_DATA(JSObjectOps) js_ObjectOps;
 
 static JSObjectOps XPC_WN_WithCall_JSOps;
 static JSObjectOps XPC_WN_NoCall_JSOps;
-static JSObjectOps XPC_WN_ModsAllowed_JSOps;
 
 /*
     Here are the enumerator cases:
@@ -1546,10 +1194,10 @@ XPC_WN_JSOp_Enumerate(JSContext *cx, JSObject *obj, JSIterateOp enum_op,
 
         rv = si->GetCallback()->
             NewEnumerate(wrapper, cx, obj, enum_op, statep, idp, &retval);
-
+        
         if(enum_op == JSENUMERATE_INIT && (NS_FAILED(rv) || !retval))
             *statep = JSVAL_NULL;
-
+        
         if(NS_FAILED(rv))
             return Throw(rv, cx);
         return retval;
@@ -1585,6 +1233,22 @@ XPC_WN_JSOp_Enumerate(JSContext *cx, JSObject *obj, JSIterateOp enum_op,
     return js_ObjectOps.enumerate(cx, obj, enum_op, statep, idp);
 }
 
+JS_STATIC_DLL_CALLBACK(void)
+XPC_WN_JSOp_Clear(JSContext *cx, JSObject *obj)
+{
+    // If our scope is cleared, make sure we clear the scope of our
+    // native wrapper as well.
+    XPCWrappedNative *wrapper =
+        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj);
+
+    if(wrapper && wrapper->IsValid())
+    {
+        XPCNativeWrapper::ClearWrappedNativeScopes(cx, wrapper);
+    }
+
+    js_ObjectOps.clear(cx, obj);
+}
+
 JSObjectOps * JS_DLL_CALLBACK
 XPC_WN_GetObjectOpsNoCall(JSContext *cx, JSClass *clazz)
 {
@@ -1603,20 +1267,14 @@ JSBool xpc_InitWrappedNativeJSOps()
     {
         memcpy(&XPC_WN_NoCall_JSOps, &js_ObjectOps, sizeof(JSObjectOps));
         XPC_WN_NoCall_JSOps.enumerate = XPC_WN_JSOp_Enumerate;
-        XPC_WN_NoCall_JSOps.defineProperty = XPC_WN_JSOp_Safe_DefineProperty;
-        XPC_WN_NoCall_JSOps.getProperty = XPC_WN_JSOp_Safe_GetProperty;
 
         memcpy(&XPC_WN_WithCall_JSOps, &js_ObjectOps, sizeof(JSObjectOps));
         XPC_WN_WithCall_JSOps.enumerate = XPC_WN_JSOp_Enumerate;
-        XPC_WN_WithCall_JSOps.defineProperty = XPC_WN_JSOp_Safe_DefineProperty;
-        XPC_WN_WithCall_JSOps.getProperty = XPC_WN_JSOp_Safe_GetProperty;
-
-        memcpy(&XPC_WN_ModsAllowed_JSOps, &js_ObjectOps, sizeof(JSObjectOps));
-        XPC_WN_ModsAllowed_JSOps.defineProperty = XPC_WN_JSOp_Safe_DefineProperty;
-        XPC_WN_ModsAllowed_JSOps.getProperty = XPC_WN_JSOp_Safe_GetProperty;
+        XPC_WN_WithCall_JSOps.clear = XPC_WN_JSOp_Clear;
 
         XPC_WN_NoCall_JSOps.call = nsnull;
         XPC_WN_NoCall_JSOps.construct = nsnull;
+        XPC_WN_NoCall_JSOps.clear = XPC_WN_JSOp_Clear;
     }
     return JS_TRUE;
 }
@@ -1664,73 +1322,74 @@ XPCNativeScriptableInfo::Construct(XPCCallContext& ccx,
 void
 XPCNativeScriptableShared::PopulateJSClass()
 {
-    NS_ASSERTION(mJSClass.name, "bad state!");
+    NS_ASSERTION(mJSClass.base.name, "bad state!");
 
-    mJSClass.flags = JSCLASS_HAS_PRIVATE |
-                     JSCLASS_PRIVATE_IS_NSISUPPORTS |
-                     JSCLASS_NEW_RESOLVE;
+    mJSClass.base.flags = JSCLASS_HAS_PRIVATE |
+                          JSCLASS_PRIVATE_IS_NSISUPPORTS |
+                          JSCLASS_NEW_RESOLVE |
+                          JSCLASS_IS_EXTENDED;
 
     if(mFlags.WantAddProperty())
-        mJSClass.addProperty = XPC_WN_Helper_AddProperty;
+        mJSClass.base.addProperty = XPC_WN_Helper_AddProperty;
     else if(mFlags.UseJSStubForAddProperty())
-        mJSClass.addProperty = JS_PropertyStub;
+        mJSClass.base.addProperty = JS_PropertyStub;
     else if(mFlags.AllowPropModsDuringResolve())
-        mJSClass.addProperty = XPC_WN_MaybeResolvingPropertyStub;
+        mJSClass.base.addProperty = XPC_WN_MaybeResolvingPropertyStub;
     else
-        mJSClass.addProperty = XPC_WN_CannotModifyPropertyStub;
+        mJSClass.base.addProperty = XPC_WN_CannotModifyPropertyStub;
 
     if(mFlags.WantDelProperty())
-        mJSClass.delProperty = XPC_WN_Helper_DelProperty;
+        mJSClass.base.delProperty = XPC_WN_Helper_DelProperty;
     else if(mFlags.UseJSStubForDelProperty())
-        mJSClass.delProperty = JS_PropertyStub;
+        mJSClass.base.delProperty = JS_PropertyStub;
     else if(mFlags.AllowPropModsDuringResolve())
-        mJSClass.delProperty = XPC_WN_MaybeResolvingPropertyStub;
+        mJSClass.base.delProperty = XPC_WN_MaybeResolvingPropertyStub;
     else
-        mJSClass.delProperty = XPC_WN_CannotModifyPropertyStub;
+        mJSClass.base.delProperty = XPC_WN_CannotModifyPropertyStub;
 
     if(mFlags.WantGetProperty())
-        mJSClass.getProperty = XPC_WN_Helper_GetProperty;
+        mJSClass.base.getProperty = XPC_WN_Helper_GetProperty;
     else
-        mJSClass.getProperty = XPC_WN_Safe_PropertyStub;
+        mJSClass.base.getProperty = JS_PropertyStub;
 
     if(mFlags.WantSetProperty())
-        mJSClass.setProperty = XPC_WN_Helper_SetProperty;
+        mJSClass.base.setProperty = XPC_WN_Helper_SetProperty;
     else if(mFlags.UseJSStubForSetProperty())
-        mJSClass.setProperty = JS_PropertyStub;
+        mJSClass.base.setProperty = JS_PropertyStub;
     else if(mFlags.AllowPropModsDuringResolve())
-        mJSClass.setProperty = XPC_WN_MaybeResolvingPropertyStub;
+        mJSClass.base.setProperty = XPC_WN_MaybeResolvingPropertyStub;
     else
-        mJSClass.setProperty = XPC_WN_CannotModifyPropertyStub;
+        mJSClass.base.setProperty = XPC_WN_CannotModifyPropertyStub;
 
     // We figure out most of the enumerate strategy at call time.
 
     if(mFlags.WantNewEnumerate() || mFlags.WantEnumerate() ||
        mFlags.DontEnumStaticProps())
-        mJSClass.enumerate = JS_EnumerateStub;
+        mJSClass.base.enumerate = JS_EnumerateStub;
     else
-        mJSClass.enumerate = XPC_WN_Shared_Enumerate;
+        mJSClass.base.enumerate = XPC_WN_Shared_Enumerate;
 
     // We have to figure out resolve strategy at call time
-    mJSClass.resolve = (JSResolveOp) XPC_WN_Helper_NewResolve;
+    mJSClass.base.resolve = (JSResolveOp) XPC_WN_Helper_NewResolve;
 
     if(mFlags.WantConvert())
-        mJSClass.convert = XPC_WN_Helper_Convert;
+        mJSClass.base.convert = XPC_WN_Helper_Convert;
     else
-        mJSClass.convert = XPC_WN_Shared_Convert;
+        mJSClass.base.convert = XPC_WN_Shared_Convert;
 
     if(mFlags.WantFinalize())
-        mJSClass.finalize = XPC_WN_Helper_Finalize;
+        mJSClass.base.finalize = XPC_WN_Helper_Finalize;
     else
-        mJSClass.finalize = XPC_WN_NoHelper_Finalize;
+        mJSClass.base.finalize = XPC_WN_NoHelper_Finalize;
 
     // We let the rest default to nsnull unless the helper wants them...
     if(mFlags.WantCheckAccess())
-        mJSClass.checkAccess = XPC_WN_Helper_CheckAccess;
+        mJSClass.base.checkAccess = XPC_WN_Helper_CheckAccess;
 
     // Note that we *must* set
-    //   mJSClass.getObjectOps = XPC_WN_GetObjectOpsNoCall
+    //   mJSClass.base.getObjectOps = XPC_WN_GetObjectOpsNoCall
     // or
-    //   mJSClass.getObjectOps = XPC_WN_GetObjectOpsWithCall
+    //   mJSClass.base.getObjectOps = XPC_WN_GetObjectOpsWithCall
     // (even for the cases were it does not do much) because with these
     // dynamically generated JSClasses, the code in
     // XPCWrappedNative::GetWrappedNativeOfJSObject() needs to look for
@@ -1739,24 +1398,28 @@ XPCNativeScriptableShared::PopulateJSClass()
 
     if(mFlags.WantCall() || mFlags.WantConstruct())
     {
-        mJSClass.getObjectOps = XPC_WN_GetObjectOpsWithCall;
+        mJSClass.base.getObjectOps = XPC_WN_GetObjectOpsWithCall;
         if(mFlags.WantCall())
-            mJSClass.call = XPC_WN_Helper_Call;
+            mJSClass.base.call = XPC_WN_Helper_Call;
         if(mFlags.WantConstruct())
-            mJSClass.construct = XPC_WN_Helper_Construct;
+            mJSClass.base.construct = XPC_WN_Helper_Construct;
     }
     else
     {
-        mJSClass.getObjectOps = XPC_WN_GetObjectOpsNoCall;
+        mJSClass.base.getObjectOps = XPC_WN_GetObjectOpsNoCall;
     }
 
     if(mFlags.WantHasInstance())
-        mJSClass.hasInstance = XPC_WN_Helper_HasInstance;
+        mJSClass.base.hasInstance = XPC_WN_Helper_HasInstance;
 
     if(mFlags.WantMark())
-        mJSClass.mark = XPC_WN_Helper_Mark;
+        mJSClass.base.mark = XPC_WN_Helper_Mark;
     else
-        mJSClass.mark = XPC_WN_Shared_Mark;
+        mJSClass.base.mark = XPC_WN_Shared_Mark;
+
+    mJSClass.equality = XPC_WN_Equality;
+    mJSClass.outerObject = XPC_WN_OuterObject;
+    mJSClass.innerObject = XPC_WN_InnerObject;
 }
 
 /***************************************************************************/
@@ -1911,12 +1574,6 @@ XPC_WN_ModsAllowed_Proto_Resolve(JSContext *cx, JSObject *obj, jsval idval)
                                  enumFlag, nsnull);
 }
 
-JS_STATIC_DLL_CALLBACK(JSObjectOps *)
-XPC_WN_ModsAllowed_Proto_GetObjectOps(JSContext *cx, JSClass *clasp)
-{
-    return &XPC_WN_ModsAllowed_JSOps;
-}
-
 
 JSClass XPC_WN_ModsAllowed_Proto_JSClass = {
     "XPC_WN_ModsAllowed_Proto_JSClass", // name;
@@ -1925,7 +1582,7 @@ JSClass XPC_WN_ModsAllowed_Proto_JSClass = {
     /* Mandatory non-null function pointer members. */
     JS_PropertyStub,                // addProperty;
     JS_PropertyStub,                // delProperty;
-    XPC_WN_Safe_PropertyStub,       // getProperty;
+    JS_PropertyStub,                // getProperty;
     JS_PropertyStub,                // setProperty;
     XPC_WN_Shared_Proto_Enumerate,         // enumerate;
     XPC_WN_ModsAllowed_Proto_Resolve,      // resolve;
@@ -1933,7 +1590,7 @@ JSClass XPC_WN_ModsAllowed_Proto_JSClass = {
     XPC_WN_Shared_Proto_Finalize,          // finalize;
 
     /* Optionally non-null members start here. */
-    XPC_WN_ModsAllowed_Proto_GetObjectOps, // getObjectOps;
+    nsnull,                         // getObjectOps;
     nsnull,                         // checkAccess;
     nsnull,                         // call;
     nsnull,                         // construct;
@@ -2006,11 +1663,6 @@ JSClass XPC_WN_NoMods_Proto_JSClass = {
     /* Mandatory non-null function pointer members. */
     XPC_WN_OnlyIWrite_Proto_PropertyStub,  // addProperty;
     XPC_WN_CannotModifyPropertyStub,       // delProperty;
-    /*
-       It's ok to use JS_PropertyStub here since this class (which is
-       used for wrapped native's prototypes) doesn't allow any changes
-       and thus no properties on the prototype can be overridden.
-     */
     JS_PropertyStub,                       // getProperty;
     XPC_WN_OnlyIWrite_Proto_PropertyStub,  // setProperty;
     XPC_WN_Shared_Proto_Enumerate,         // enumerate;
@@ -2094,7 +1746,7 @@ JSClass XPC_WN_Tearoff_JSClass = {
     /* Mandatory non-null function pointer members. */
     XPC_WN_OnlyIWrite_PropertyStub,     // addProperty;
     XPC_WN_CannotModifyPropertyStub,    // delProperty;
-    XPC_WN_Safe_PropertyStub,           // getProperty;
+    JS_PropertyStub,                    // getProperty;
     XPC_WN_OnlyIWrite_PropertyStub,     // setProperty;
     XPC_WN_TearOff_Enumerate,           // enumerate;
     XPC_WN_TearOff_Resolve,             // resolve;
