@@ -1,38 +1,41 @@
-/* 
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
  * The Original Code is the Netscape security libraries.
- * 
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are 
- * Copyright (C) 1994-2000 Netscape Communications Corporation.  All
- * Rights Reserved.
- * 
+ *
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1994-2000
+ * the Initial Developer. All Rights Reserved.
+ *
  * Contributor(s):
- * 
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU General Public License Version 2 or later (the
- * "GPL"), in which case the provisions of the GPL are applicable 
- * instead of those above.  If you wish to allow use of your 
- * version of this file only under the terms of the GPL and not to
- * allow others to use your version of this file under the MPL,
- * indicate your decision by deleting the provisions above and
- * replace them with the notice and other provisions required by
- * the GPL.  If you do not delete the provisions above, a recipient
- * may use your version of this file under either the MPL or the
- * GPL.
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.49 $ $Date: 2003/11/15 00:09:51 $ $Name: THUNDERBIRD_1_0_7_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.53 $ $Date: 2005/01/20 02:25:48 $";
 #endif /* DEBUG */
 
 #ifndef NSSPKI_H
@@ -401,7 +404,9 @@ find_cert_issuer (
   NSSCertificate *c,
   NSSTime *timeOpt,
   NSSUsage *usage,
-  NSSPolicies *policiesOpt
+  NSSPolicies *policiesOpt,
+  NSSTrustDomain *td,
+  NSSCryptoContext *cc
 )
 {
     NSSArena *arena;
@@ -409,15 +414,11 @@ find_cert_issuer (
     NSSCertificate **ccIssuers = NULL;
     NSSCertificate **tdIssuers = NULL;
     NSSCertificate *issuer = NULL;
-    NSSTrustDomain *td;
-    NSSCryptoContext *cc;
-    cc = c->object.cryptoContext; /* NSSCertificate_GetCryptoContext(c); */
-    td = NSSCertificate_GetTrustDomain(c);
-#ifdef NSS_3_4_CODE
-    if (!td) {
-	td = STAN_GetDefaultTrustDomain();
-    }
-#endif
+
+    if (!cc)
+	cc = c->object.cryptoContext;
+    if (!td)
+	td = NSSCertificate_GetTrustDomain(c);
     arena = nssArena_Create();
     if (!arena) {
 	return (NSSCertificate *)NULL;
@@ -429,7 +430,8 @@ find_cert_issuer (
 	                                                       0,
 	                                                       arena);
     }
-    tdIssuers = nssTrustDomain_FindCertificatesBySubject(td,
+    if (td)
+	tdIssuers = nssTrustDomain_FindCertificatesBySubject(td,
                                                          &c->issuer,
                                                          NULL,
                                                          0,
@@ -442,6 +444,10 @@ find_cert_issuer (
 	if (dc) {
 	    issuerID = dc->getIssuerIdentifier(dc);
 	}
+	/* XXX review based on CERT_FindCertIssuer
+	 * this function is not using the authCertIssuer field as a fallback
+	 * if authority key id does not exist
+	 */
 	if (issuerID) {
 	    certs = filter_subject_certs_for_id(certs, issuerID);
 	}
@@ -456,10 +462,9 @@ find_cert_issuer (
     return issuer;
 }
 
-/* XXX review based on CERT_FindCertIssuer
- * this function is not using the authCertIssuer field as a fallback
- * if authority key id does not exist
- */
+/* This function returns the built chain, as far as it gets,
+** even if/when it fails to find an issuer, and returns PR_FAILURE
+*/
 NSS_IMPLEMENT NSSCertificate **
 nssCertificate_BuildChain (
   NSSCertificate *c,
@@ -469,72 +474,75 @@ nssCertificate_BuildChain (
   NSSCertificate **rvOpt,
   PRUint32 rvLimit,
   NSSArena *arenaOpt,
-  PRStatus *statusOpt
+  PRStatus *statusOpt,
+  NSSTrustDomain *td,
+  NSSCryptoContext *cc 
 )
 {
-    NSSCertificate **rvChain;
-#ifdef NSS_3_4_CODE
-    NSSCertificate *cp;
-    CERTCertificate *cCert;
-#endif
+    NSSCertificate **rvChain = NULL;
     NSSUsage issuerUsage = *usage;
-    NSSTrustDomain *td;
     nssPKIObjectCollection *collection;
+    PRUint32  rvCount = 0;
+    PRStatus  st;
+    PRStatus  ret = PR_SUCCESS;
 
-    td = NSSCertificate_GetTrustDomain(c);
+    if (!td)
+	td = NSSCertificate_GetTrustDomain(c);
+    if (!td || !c || !cc) 
+	goto loser;
 #ifdef NSS_3_4_CODE
-    if (!td) {
-	td = STAN_GetDefaultTrustDomain();
-    }
     /* bump the usage up to CA level */
     issuerUsage.nss3lookingForCA = PR_TRUE;
 #endif
-    if (statusOpt) *statusOpt = PR_SUCCESS;
     collection = nssCertificateCollection_Create(td, NULL);
-    if (!collection) {
-	if (statusOpt) *statusOpt = PR_FAILURE;
-	return (NSSCertificate **)NULL;
-    }
-    nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
-    if (rvLimit == 1) {
-	goto finish;
-    }
+    if (!collection)
+	goto loser;
+    st = nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
+    if (st != PR_SUCCESS)
+    	goto loser;
     /* XXX This breaks code for which NSS_3_4_CODE is not defined (pure
      *     4.0 builds).  That won't affect the tip.  But be careful
      *     when merging 4.0!!!
      */
-    while (c != (NSSCertificate *)NULL) {
+    for (rvCount = 1; (!rvLimit || rvCount < rvLimit); ++rvCount) {
 #ifdef NSS_3_4_CODE
-	cCert = STAN_GetCERTCertificate(c);
+	CERTCertificate *cCert = STAN_GetCERTCertificate(c);
 	if (cCert->isRoot) {
 	    /* not including the issuer of the self-signed cert, which is,
 	     * of course, itself
 	     */
 	    break;
 	}
-	cp = c;
 #endif
-	c = find_cert_issuer(c, timeOpt, &issuerUsage, policiesOpt);
-	if (c) {
-	    nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
-	    nssCertificate_Destroy(c); /* collection has it */
-	    if (rvLimit > 0 &&
-	        nssPKIObjectCollection_Count(collection) == rvLimit) 
-	    {
-		break;
-	    }
-	} else {
-	    nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
-	    if (statusOpt) *statusOpt = PR_FAILURE;
+	c = find_cert_issuer(c, timeOpt, &issuerUsage, policiesOpt, td, cc);
+	if (!c) {
+	    ret = PR_FAILURE;
 	    break;
 	}
+	st = nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
+	nssCertificate_Destroy(c); /* collection has it */
+	if (st != PR_SUCCESS)
+	    goto loser;
     }
-finish:
     rvChain = nssPKIObjectCollection_GetCertificates(collection, 
                                                      rvOpt, 
                                                      rvLimit, 
                                                      arenaOpt);
-    nssPKIObjectCollection_Destroy(collection);
+    if (rvChain) {
+	nssPKIObjectCollection_Destroy(collection);
+	if (statusOpt) 
+	    *statusOpt = ret;
+	if (ret != PR_SUCCESS)
+	    nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
+	return rvChain;
+    }
+
+loser:
+    if (collection)
+	nssPKIObjectCollection_Destroy(collection);
+    if (statusOpt) 
+	*statusOpt = PR_FAILURE;
+    nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
     return rvChain;
 }
 
@@ -547,11 +555,14 @@ NSSCertificate_BuildChain (
   NSSCertificate **rvOpt,
   PRUint32 rvLimit, /* zero for no limit */
   NSSArena *arenaOpt,
-  PRStatus *statusOpt
+  PRStatus *statusOpt,
+  NSSTrustDomain *td,
+  NSSCryptoContext *cc 
 )
 {
     return nssCertificate_BuildChain(c, timeOpt, usage, policiesOpt,
-                                     rvOpt, rvLimit, arenaOpt, statusOpt);
+                                     rvOpt, rvLimit, arenaOpt, statusOpt,
+				     td, cc);
 }
 
 NSS_IMPLEMENT NSSCryptoContext *
@@ -1019,6 +1030,8 @@ nssTrust_Create (
     nssCryptokiObject *instance;
     nssTrustLevel serverAuth, clientAuth, codeSigning, emailProtection;
     SECStatus rv; /* Should be stan flavor */
+    PRBool stepUp;
+
     lastTrustOrder = 1<<16; /* just make it big */
     PR_ASSERT(object->instances != NULL && object->numInstances > 0);
     rvt = nss_ZNEW(object->arena, NSSTrust);
@@ -1044,7 +1057,8 @@ nssTrust_Create (
 	                                        &serverAuth,
 	                                        &clientAuth,
 	                                        &codeSigning,
-	                                        &emailProtection);
+	                                        &emailProtection,
+	                                        &stepUp);
 	if (status != PR_SUCCESS) {
 	    PZ_Unlock(object->lock);
 	    return (NSSTrust *)NULL;
@@ -1073,6 +1087,7 @@ nssTrust_Create (
 	{
 	    rvt->codeSigning = codeSigning;
 	}
+	rvt->stepUpApproved = stepUp;
 	lastTrustOrder = myTrustOrder;
     }
     PZ_Unlock(object->lock);
